@@ -8,8 +8,10 @@
 # or in the "LICENSE.txt" file accompanying this file. This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES
 # OR CONDITIONS OF ANY KIND, express or implied. See the License for the specific language governing permissions and
 # limitations under the License.
+import itertools
 import re
-from typing import List
+from datetime import datetime
+from typing import Any, List, Tuple
 
 from botocore.exceptions import ClientError
 
@@ -23,7 +25,6 @@ from pcluster.constants import (
     OS_TO_IMAGE_NAME_PART_MAP,
     PCLUSTER_IMAGE_BUILD_STATUS_TAG,
     PCLUSTER_IMAGE_ID_TAG,
-    SUPPORTED_ARCHITECTURES,
 )
 from pcluster.utils import get_partition
 
@@ -36,6 +37,7 @@ class Ec2Client(Boto3Client):
         self.additional_instance_types_data = {}
         self.security_groups_cache = {}
         self.subnets_cache = {}
+        self.capacity_reservations_cache = {}
 
     @AWSExceptionHandler.handle_client_exception
     @Cache.cached
@@ -84,6 +86,30 @@ class Ec2Client(Boto3Client):
                 result.append(subnet)
         return result
 
+    @AWSExceptionHandler.handle_client_exception
+    def describe_capacity_reservations(self, capacity_reservation_ids):
+        """Return a list of Capacity Reservations."""
+        result = []
+        missed_capacity_reservations = []
+        for capacity_reservation_id in capacity_reservation_ids:
+            cached_data = self.capacity_reservations_cache.get(capacity_reservation_id)
+            if cached_data:
+                result.append(cached_data)
+            else:
+                missed_capacity_reservations.append(capacity_reservation_id)
+        if missed_capacity_reservations:
+            response = list(
+                self._paginate_results(
+                    self._client.describe_capacity_reservations, CapacityReservationIds=missed_capacity_reservations
+                )
+            )
+            for capacity_reservation in response:
+                self.capacity_reservations_cache[
+                    capacity_reservation.get("CapacityReservationId")
+                ] = capacity_reservation
+                result.append(capacity_reservation)
+        return result
+
     @AWSExceptionHandler.handle_client_exception
     @Cache.cached
     def get_subnet_avail_zone(self, subnet_id):
@@ -102,6 +128,15 @@ class Ec2Client(Boto3Client):
             return subnets[0].get("VpcId")
         raise AWSClientError(function_name="describe_subnets", message=f"Subnet {subnet_id} not found")
 
+    @AWSExceptionHandler.handle_client_exception
+    @Cache.cached
+    def get_subnet_cidr(self, subnet_id):
+        """Return cidr block  of the given subnet."""
+        subnets = self._client.describe_subnets(SubnetIds=[subnet_id]).get("Subnets")
+        if subnets:
+            return subnets[0].get("CidrBlock")
+        raise AWSClientError(function_name="describe_subnets", message=f"Subnet {subnet_id} not found")
+
     @AWSExceptionHandler.handle_client_exception
     @Cache.cached
     def describe_image(self, ami_id):
@@ -244,10 +279,14 @@ class Ec2Client(Boto3Client):
     def get_supported_architectures(self, instance_type):
         """Return a list of architectures supported for the given instance type."""
         instance_info = self.get_instance_type_info(instance_type)
-        supported_architectures = instance_info.supported_architecture()
+        return instance_info.supported_architecture()
 
-        # Some instance types support multiple architectures (x86_64 and i386). Filter unsupported ones.
-        return list(set(supported_architectures) & set(SUPPORTED_ARCHITECTURES))
+    @staticmethod
+    def _is_image_deprecated(image):
+        return "DeprecationTime" in image and utils.to_iso_timestr(datetime.now()) >= image["DeprecationTime"]
+
+    def _find_valid_official_image(self, images):
+        return max(images, key=lambda image: ("0" if self._is_image_deprecated(image) else "1") + image["CreationDate"])
 
     @AWSExceptionHandler.handle_client_exception
     @Cache.cached
@@ -258,20 +297,25 @@ class Ec2Client(Boto3Client):
 
         filters = [{"Name": "name", "Values": ["{0}*".format(self._get_official_image_name_prefix(os, architecture))]}]
         filters.extend([{"Name": f"tag:{tag.key}", "Values": [tag.value]} for tag in tags])
-        images = self._client.describe_images(Owners=[owner], Filters=filters).get("Images")
+        images = self._client.describe_images(Owners=[owner], Filters=filters, IncludeDeprecated=True).get("Images")
         if not images:
             raise AWSClientError(function_name="describe_images", message="Cannot find official ParallelCluster AMI")
-        return max(images, key=lambda image: image["CreationDate"]).get("ImageId")
+        return self._find_valid_official_image(images).get("ImageId")
 
+    @AWSExceptionHandler.handle_client_exception
+    @Cache.cached
     def get_official_images(self, os=None, architecture=None):
         """Get the list of official images, optionally filtered by os and architecture."""
-        try:
-            owners = ["amazon"]
-            name = f"{self._get_official_image_name_prefix(os, architecture)}*"
-            filters = [{"Name": "name", "Values": [name]}]
-            return self.describe_images(ami_ids=[], owners=owners, filters=filters)
-        except ImageNotFoundError:
-            return []
+        owners = ["amazon"]
+        name = f"{self._get_official_image_name_prefix(os, architecture)}*"
+        filters = [{"Name": "name", "Values": [name]}]
+        return [
+            ImageInfo(self._find_valid_official_image(images_os_arch))
+            for _, images_os_arch in itertools.groupby(
+                self._client.describe_images(Owners=owners, Filters=filters, IncludeDeprecated=True).get("Images"),
+                key=lambda image: f'{self.extract_os_from_official_image_name(image["Name"])}-{image["Architecture"]}',
+            )
+        ]
 
     @AWSExceptionHandler.handle_client_exception
     @Cache.cached
@@ -310,7 +354,7 @@ class Ec2Client(Boto3Client):
         ]
 
     @AWSExceptionHandler.handle_client_exception
-    def describe_instances(self, filters, next_token=None):
+    def describe_instances(self, filters, next_token=None) -> Tuple[List[Any], str]:
         """Retrieve a filtered list of instances."""
         describe_stacks_kwargs = {}
         if next_token:
@@ -432,7 +476,7 @@ class Ec2Client(Boto3Client):
 
     @AWSExceptionHandler.handle_client_exception
     def run_instances(self, **kwargs):
-        """Describe network interfaces."""
+        """Run instances."""
         try:
             self._client.run_instances(**kwargs)
         except ClientError as e:
