@@ -210,7 +210,9 @@ def configure(args):  # noqa: C901
                         default_value=default_instance_type,
                     )
                     if compute_instance_type not in [
-                        compute_resource["InstanceType"] for compute_resource in compute_resources
+                        instances["InstanceType"]
+                        for compute_resource in compute_resources
+                        for instances in compute_resource["Instances"]
                     ]:
                         break
                     print(
@@ -245,7 +247,7 @@ def configure(args):  # noqa: C901
             else:
                 compute_resource = {
                     "Name": compute_resource_name,
-                    "InstanceType": compute_instance_type,
+                    "Instances": [{"InstanceType": compute_instance_type}],
                     "MinCount": min_cluster_size,
                     "MaxCount": max_cluster_size,
                 }
