@@ -25,7 +25,6 @@ from typing import List, Optional, Set, Tuple
 from urllib.request import urlopen
 
 import pkg_resources
-import yaml
 from jinja2 import BaseLoader
 from jinja2.sandbox import SandboxedEnvironment
 from marshmallow import ValidationError
@@ -67,8 +66,15 @@ from pcluster.models.compute_fleet_status_manager import ComputeFleetStatus, Com
 from pcluster.models.s3_bucket import S3Bucket, S3BucketFactory, S3FileFormat, create_s3_presigned_url, parse_bucket_url
 from pcluster.schemas.cluster_schema import ClusterSchema
 from pcluster.templates.cdk_builder import CDKTemplateBuilder
-from pcluster.utils import datetime_to_epoch, generate_random_name_with_prefix, get_attr, get_installed_version, grouper
-from pcluster.validators.common import FailureLevel, ValidationResult
+from pcluster.utils import (
+    datetime_to_epoch,
+    generate_random_name_with_prefix,
+    get_attr,
+    get_installed_version,
+    grouper,
+    yaml_load,
+)
+from pcluster.validators.common import FailureLevel, ValidationResult, ValidatorContext
 
 # pylint: disable=C0302
 
@@ -400,7 +406,9 @@ class Cluster:
         """Validate a create cluster request."""
         self._validate_no_existing_stack()
         self.config, ignored_validation_failures = self._validate_and_parse_config(
-            validator_suppressors, validation_failure_level
+            validator_suppressors=validator_suppressors,
+            validation_failure_level=validation_failure_level,
+            context=ValidatorContext(),
         )
         if dry_run and isinstance(self.config.scheduling, SchedulerPluginScheduling):
             self._render_and_upload_scheduler_plugin_template(dry_run=dry_run)
@@ -410,7 +418,9 @@ class Cluster:
         if AWSApi.instance().cfn.stack_exists(self.stack_name):
             raise BadRequestClusterActionError(f"Cluster {self.name} already exists.")
 
-    def _validate_and_parse_config(self, validator_suppressors, validation_failure_level, config_text=None):
+    def _validate_and_parse_config(
+        self, validator_suppressors, validation_failure_level, config_text=None, context: ValidatorContext = None
+    ):
         """
         Perform syntactic and semantic validation and return parsed config.
 
@@ -422,7 +432,7 @@ class Cluster:
             LOGGER.info("Validating cluster configuration...")
             Cluster._load_additional_instance_type_data(cluster_config_dict)
             config = self._load_config(cluster_config_dict)
-            validation_failures = config.validate(validator_suppressors)
+            validation_failures = config.validate(validator_suppressors, context)
             if any(f.level.value >= FailureLevel(validation_failure_level).value for f in validation_failures):
                 raise ConfigValidationError("Invalid cluster configuration.", validation_failures=validation_failures)
             LOGGER.info("Validation succeeded.")
@@ -651,7 +661,7 @@ class Cluster:
     def _get_stack_template(self):
         """Return the template body of the stack."""
         try:
-            return yaml.safe_load(AWSApi.instance().cfn.get_stack_template(self.stack_name))
+            return yaml_load(AWSApi.instance().cfn.get_stack_template(self.stack_name))
         except AWSClientError as e:
             raise _cluster_error_mapper(e, f"Unable to retrieve template for stack {self.stack_name}. {e}")
 
@@ -698,7 +708,9 @@ class Cluster:
             filters.append({"Name": f"tag:{PCLUSTER_QUEUE_NAME_TAG}", "Values": [queue_name]})
         return filters
 
-    def describe_instances(self, node_type: NodeType = None, next_token: str = None, queue_name: str = None):
+    def describe_instances(
+        self, node_type: NodeType = None, next_token: str = None, queue_name: str = None
+    ) -> Tuple[List[ClusterInstance], str]:
         """Return the cluster instances filtered by node type."""
         try:
             filters = self._get_instance_filters(node_type, queue_name)
@@ -818,7 +830,10 @@ class Cluster:
         self._validate_cluster_exists()
         self._validate_stack_status_not_in_progress()
         target_config, ignored_validation_failures = self._validate_and_parse_config(
-            validator_suppressors, validation_failure_level, target_source_config
+            validator_suppressors=validator_suppressors,
+            validation_failure_level=validation_failure_level,
+            config_text=target_source_config,
+            context=ValidatorContext(head_node_instance_id=self.head_node_instance.id),
         )
         changes = self._validate_patch(force, target_config)
 
