@@ -18,9 +18,9 @@ import copy
 import hashlib
 import logging
 import re
+from typing import List
 from urllib.request import urlopen
 
-import yaml
 from marshmallow import ValidationError, fields, post_load, pre_dump, pre_load, validate, validates, validates_schema
 from yaml import YAMLError
 
@@ -28,6 +28,7 @@ from pcluster.aws.aws_api import AWSApi
 from pcluster.aws.common import AWSClientError
 from pcluster.config.cluster_config import (
     AdditionalPackages,
+    AllocationStrategy,
     AmiSearchFilters,
     AwsBatchClusterConfig,
     AwsBatchComputeResource,
@@ -35,6 +36,7 @@ from pcluster.config.cluster_config import (
     AwsBatchQueueNetworking,
     AwsBatchScheduling,
     AwsBatchSettings,
+    CapacityReservationTarget,
     CapacityType,
     CloudWatchDashboards,
     CloudWatchLogs,
@@ -44,6 +46,7 @@ from pcluster.config.cluster_config import (
     CustomAction,
     CustomActions,
     Dashboards,
+    Database,
     Dcv,
     DirectoryService,
     Dns,
@@ -51,6 +54,7 @@ from pcluster.config.cluster_config import (
     EphemeralVolume,
     ExistingFsxOntap,
     ExistingFsxOpenZfs,
+    FlexibleInstanceType,
     HeadNode,
     HeadNodeImage,
     HeadNodeNetworking,
@@ -96,6 +100,8 @@ from pcluster.config.cluster_config import (
     SharedFsxLustre,
     SlurmClusterConfig,
     SlurmComputeResource,
+    SlurmComputeResourceNetworking,
+    SlurmFlexibleComputeResource,
     SlurmQueue,
     SlurmQueueNetworking,
     SlurmScheduling,
@@ -119,15 +125,11 @@ from pcluster.constants import (
     SUPPORTED_OSES,
 )
 from pcluster.models.s3_bucket import parse_bucket_url
-from pcluster.schemas.common_schema import (
-    AdditionalIamPolicySchema,
-    BaseDevSettingsSchema,
-    BaseSchema,
-    TagSchema,
-    get_field_validator,
-    validate_no_reserved_tag,
-)
-from pcluster.validators.cluster_validators import FSX_MESSAGES
+from pcluster.schemas.common_schema import AdditionalIamPolicySchema, BaseDevSettingsSchema, BaseSchema
+from pcluster.schemas.common_schema import ImdsSchema as TopLevelImdsSchema
+from pcluster.schemas.common_schema import TagSchema, get_field_validator, validate_no_reserved_tag
+from pcluster.utils import yaml_load
+from pcluster.validators.cluster_validators import EFS_MESSAGES, FSX_MESSAGES
 
 # pylint: disable=C0302
 
@@ -313,6 +315,21 @@ class EfsSettingsSchema(BaseSchema):
         validate=validate.Regexp(r"^fs-[0-9a-z]{8}$|^fs-[0-9a-z]{17}$"),
         metadata={"update_policy": UpdatePolicy.UNSUPPORTED},
     )
+    deletion_policy = fields.Str(
+        validate=validate.OneOf(DELETION_POLICIES), metadata={"update_policy": UpdatePolicy.SUPPORTED}
+    )
+
+    @validates_schema
+    def validate_file_system_id_ignored_parameters(self, data, **kwargs):
+        """Return errors for parameters in the Efs config section that would be ignored."""
+        # If file_system_id is specified, all parameters are ignored.
+        messages = []
+        if data.get("file_system_id") is not None:
+            for key in data:
+                if key is not None and key != "file_system_id":
+                    messages.append(EFS_MESSAGES["errors"]["ignored_param_with_efs_fs_id"].format(efs_param=key))
+            if messages:
+                raise ValidationError(message=messages)
 
     @validates_schema
     def validate_existence_of_mode_throughput(self, data, **kwargs):
@@ -363,11 +380,13 @@ class FsxLustreSettingsSchema(BaseSchema):
     )
     per_unit_storage_throughput = fields.Int(metadata={"update_policy": UpdatePolicy.UNSUPPORTED})
     backup_id = fields.Str(
-        validate=validate.Regexp("^(backup-[0-9a-f]{8,})$"), metadata={"update_policy": UpdatePolicy.UNSUPPORTED}
+        validate=validate.Regexp("^(backup-[0-9a-f]{8,})$"),
+        metadata={"update_policy": UpdatePolicy.UNSUPPORTED},
     )
     kms_key_id = fields.Str(metadata={"update_policy": UpdatePolicy.UNSUPPORTED})
     file_system_id = fields.Str(
-        validate=validate.Regexp(r"^fs-[0-9a-z]{17}$"), metadata={"update_policy": UpdatePolicy.UNSUPPORTED}
+        validate=validate.Regexp(r"^fs-[0-9a-z]{17}$"),
+        metadata={"update_policy": UpdatePolicy.UNSUPPORTED},
     )
     auto_import_policy = fields.Str(
         validate=validate.OneOf(["NEW", "NEW_CHANGED", "NEW_CHANGED_DELETED"]),
@@ -384,6 +403,9 @@ class FsxLustreSettingsSchema(BaseSchema):
         validate=validate.OneOf(["HDD", "SSD"]),
         metadata={"update_policy": UpdatePolicy.UNSUPPORTED},
     )
+    deletion_policy = fields.Str(
+        validate=validate.OneOf(DELETION_POLICIES), metadata={"update_policy": UpdatePolicy.SUPPORTED}
+    )
 
     @validates_schema
     def validate_file_system_id_ignored_parameters(self, data, **kwargs):
@@ -445,7 +467,9 @@ class SharedStorageSchema(BaseSchema):
     """Represent the generic SharedStorage schema."""
 
     mount_dir = fields.Str(
-        required=True, validate=get_field_validator("file_path"), metadata={"update_policy": UpdatePolicy.UNSUPPORTED}
+        required=True,
+        validate=get_field_validator("file_path"),
+        metadata={"update_policy": UpdatePolicy.SHARED_STORAGE_UPDATE_POLICY},
     )
     name = fields.Str(required=True, metadata={"update_policy": UpdatePolicy.UNSUPPORTED})
     storage_type = fields.Str(
@@ -453,9 +477,9 @@ class SharedStorageSchema(BaseSchema):
         validate=validate.OneOf(["Ebs", FSX_LUSTRE, FSX_OPENZFS, FSX_ONTAP, "Efs"]),
         metadata={"update_policy": UpdatePolicy.UNSUPPORTED},
     )
-    ebs_settings = fields.Nested(EbsSettingsSchema, metadata={"update_policy": UpdatePolicy.UNSUPPORTED})
-    efs_settings = fields.Nested(EfsSettingsSchema, metadata={"update_policy": UpdatePolicy.UNSUPPORTED})
-    fsx_lustre_settings = fields.Nested(FsxLustreSettingsSchema, metadata={"update_policy": UpdatePolicy.UNSUPPORTED})
+    ebs_settings = fields.Nested(EbsSettingsSchema, metadata={"update_policy": UpdatePolicy.IGNORED})
+    efs_settings = fields.Nested(EfsSettingsSchema, metadata={"update_policy": UpdatePolicy.IGNORED})
+    fsx_lustre_settings = fields.Nested(FsxLustreSettingsSchema, metadata={"update_policy": UpdatePolicy.IGNORED})
     fsx_open_zfs_settings = fields.Nested(
         FsxOpenZfsSettingsSchema, metadata={"update_policy": UpdatePolicy.UNSUPPORTED}
     )
@@ -608,8 +632,9 @@ class HeadNodeNetworkingSchema(BaseNetworkingSchema):
 class PlacementGroupSchema(BaseSchema):
     """Represent the schema of placement group."""
 
-    enabled = fields.Bool(metadata={"update_policy": UpdatePolicy.QUEUE_UPDATE_STRATEGY})
-    id = fields.Str(metadata={"update_policy": UpdatePolicy.QUEUE_UPDATE_STRATEGY})
+    enabled = fields.Bool(metadata={"update_policy": UpdatePolicy.MANAGED_PLACEMENT_GROUP})
+    id = fields.Str(metadata={"update_policy": UpdatePolicy.MANAGED_PLACEMENT_GROUP})
+    name = fields.Str(metadata={"update_policy": UpdatePolicy.MANAGED_PLACEMENT_GROUP})
 
     @post_load
     def make_resource(self, data, **kwargs):
@@ -624,7 +649,7 @@ class QueueNetworkingSchema(BaseNetworkingSchema):
         fields.Str(validate=get_field_validator("subnet_id")),
         required=True,
         validate=validate.Length(equal=1),
-        metadata={"update_policy": UpdatePolicy.QUEUE_UPDATE_STRATEGY},
+        metadata={"update_policy": UpdatePolicy.MANAGED_FSX},
     )
     assign_public_ip = fields.Bool(metadata={"update_policy": UpdatePolicy.UNSUPPORTED})
 
@@ -633,7 +658,7 @@ class SlurmQueueNetworkingSchema(QueueNetworkingSchema):
     """Represent the schema of the Networking, child of slurm Queue."""
 
     placement_group = fields.Nested(
-        PlacementGroupSchema, metadata={"update_policy": UpdatePolicy.QUEUE_UPDATE_STRATEGY}
+        PlacementGroupSchema, metadata={"update_policy": UpdatePolicy.MANAGED_PLACEMENT_GROUP}
     )
     proxy = fields.Nested(QueueProxySchema, metadata={"update_policy": UpdatePolicy.QUEUE_UPDATE_STRATEGY})
 
@@ -925,6 +950,32 @@ class TimeoutsSchema(BaseSchema):
         return Timeouts(**data)
 
 
+class CapacityReservationTargetSchema(BaseSchema):
+    """Represent the schema of the CapacityReservationTarget section."""
+
+    capacity_reservation_id = fields.Str(metadata={"update_policy": UpdatePolicy.QUEUE_UPDATE_STRATEGY})
+    capacity_reservation_resource_group_arn = fields.Str(metadata={"update_policy": UpdatePolicy.QUEUE_UPDATE_STRATEGY})
+
+    @post_load()
+    def make_resource(self, data, **kwargs):
+        """Generate resource."""
+        return CapacityReservationTarget(**data)
+
+    @validates_schema
+    def no_coexist_id_and_group_arn(self, data, **kwargs):
+        """Validate that 'capacity_reservation_id' and 'capacity_reservation_resource_group_arn' do not co-exist."""
+        if self.fields_coexist(
+            data,
+            ["capacity_reservation_id", "capacity_reservation_resource_group_arn"],
+            one_required=True,
+            **kwargs,
+        ):
+            raise ValidationError(
+                "A Capacity Reservation Target needs to specify either Capacity Reservation ID or "
+                "Capacity Reservation Resource Group ARN."
+            )
+
+
 class ClusterDevSettingsSchema(BaseDevSettingsSchema):
     """Represent the schema of Dev Setting."""
 
@@ -1039,6 +1090,17 @@ class QueueCustomActionsSchema(BaseSchema):
         return CustomActions(**data)
 
 
+class InstanceTypeSchema(BaseSchema):
+    """Schema of a compute resource that supports a pool of instance types."""
+
+    instance_type = fields.Str(required=True, metadata={"update_policy": UpdatePolicy.COMPUTE_FLEET_STOP})
+
+    @post_load
+    def make_resource(self, data, **kwargs):
+        """Generate resource."""
+        return FlexibleInstanceType(**data)
+
+
 class HeadNodeSchema(BaseSchema):
     """Represent the schema of the HeadNode."""
 
@@ -1067,10 +1129,28 @@ class _ComputeResourceSchema(BaseSchema):
     name = fields.Str(required=True, metadata={"update_policy": UpdatePolicy.UNSUPPORTED})
 
 
+class SlurmComputeResourceNetworkingSchema(BaseSchema):
+    """Represent the Networking schema of the Slurm ComputeResource."""
+
+    placement_group = fields.Nested(
+        PlacementGroupSchema, metadata={"update_policy": UpdatePolicy.MANAGED_PLACEMENT_GROUP}
+    )
+
+    @post_load
+    def make_resource(self, data, **kwargs):
+        """Generate resource."""
+        return SlurmComputeResourceNetworking(**data)
+
+
 class SlurmComputeResourceSchema(_ComputeResourceSchema):
     """Represent the schema of the Slurm ComputeResource."""
 
-    instance_type = fields.Str(required=True, metadata={"update_policy": UpdatePolicy.COMPUTE_FLEET_STOP})
+    instance_type = fields.Str(metadata={"update_policy": UpdatePolicy.COMPUTE_FLEET_STOP})
+    instances = fields.Nested(
+        InstanceTypeSchema,
+        many=True,
+        metadata={"update_policy": UpdatePolicy.COMPUTE_FLEET_STOP_ON_REMOVE, "update_key": "InstanceType"},
+    )
     max_count = fields.Int(validate=validate.Range(min=1), metadata={"update_policy": UpdatePolicy.MAX_COUNT})
     min_count = fields.Int(validate=validate.Range(min=0), metadata={"update_policy": UpdatePolicy.COMPUTE_FLEET_STOP})
     spot_price = fields.Float(
@@ -1079,10 +1159,42 @@ class SlurmComputeResourceSchema(_ComputeResourceSchema):
     efa = fields.Nested(EfaSchema, metadata={"update_policy": UpdatePolicy.QUEUE_UPDATE_STRATEGY})
     disable_simultaneous_multithreading = fields.Bool(metadata={"update_policy": UpdatePolicy.COMPUTE_FLEET_STOP})
     schedulable_memory = fields.Int(metadata={"update_policy": UpdatePolicy.QUEUE_UPDATE_STRATEGY})
+    capacity_reservation_target = fields.Nested(
+        CapacityReservationTargetSchema, metadata={"update_policy": UpdatePolicy.QUEUE_UPDATE_STRATEGY}
+    )
+    networking = fields.Nested(
+        SlurmComputeResourceNetworkingSchema, metadata={"update_policy": UpdatePolicy.MANAGED_PLACEMENT_GROUP}
+    )
+
+    @validates_schema
+    def no_coexist_instance_type_flexibility(self, data, **kwargs):
+        """Validate that 'instance_type' and 'instances' do not co-exist."""
+        if self.fields_coexist(
+            data,
+            ["instance_type", "instances"],
+            one_required=True,
+            **kwargs,
+        ):
+            raise ValidationError("A Compute Resource needs to specify either InstanceType or Instances.")
+
+    @validates("instances")
+    def no_duplicate_instance_types(self, flexible_instance_types: List[FlexibleInstanceType]):
+        """Verify that there are no duplicates in Instances."""
+        instance_types = set()
+        for flexible_instance_type in flexible_instance_types:
+            instance_type_name = flexible_instance_type.instance_type
+            if instance_type_name in instance_types:
+                raise ValidationError(
+                    f"Duplicate instance type ({instance_type_name}) detected. Instances should not have "
+                    f"duplicate instance types. "
+                )
+            instance_types.add(instance_type_name)
 
     @post_load
     def make_resource(self, data, **kwargs):
         """Generate resource."""
+        if data.get("instances"):
+            return SlurmFlexibleComputeResource(**data)
         return SlurmComputeResource(**data)
 
 
@@ -1156,11 +1268,18 @@ class _CommonQueueSchema(BaseQueueSchema):
     )
     iam = fields.Nested(QueueIamSchema, metadata={"update_policy": UpdatePolicy.SUPPORTED})
     image = fields.Nested(QueueImageSchema, metadata={"update_policy": UpdatePolicy.QUEUE_UPDATE_STRATEGY})
+    capacity_reservation_target = fields.Nested(
+        CapacityReservationTargetSchema, metadata={"update_policy": UpdatePolicy.QUEUE_UPDATE_STRATEGY}
+    )
 
 
 class SlurmQueueSchema(_CommonQueueSchema):
     """Represent the schema of a Slurm Queue."""
 
+    allocation_strategy = fields.Str(
+        validate=validate.OneOf([strategy.value for strategy in AllocationStrategy]),
+        metadata={"update_policy": UpdatePolicy.QUEUE_UPDATE_STRATEGY},
+    )
     compute_resources = fields.Nested(
         SlurmComputeResourceSchema,
         many=True,
@@ -1227,6 +1346,23 @@ class DnsSchema(BaseSchema):
         return Dns(**data)
 
 
+class DatabaseSchema(BaseSchema):
+    """Represent the schema of the DirectoryService."""
+
+    uri = fields.Str(required=True, metadata={"update_policy": UpdatePolicy.COMPUTE_FLEET_STOP})
+    user_name = fields.Str(required=True, metadata={"update_policy": UpdatePolicy.COMPUTE_FLEET_STOP})
+    password_secret_arn = fields.Str(
+        required=True,
+        validate=validate.Regexp(r"^arn:.*:secret"),
+        metadata={"update_policy": UpdatePolicy.COMPUTE_FLEET_STOP},
+    )
+
+    @post_load
+    def make_resource(self, data, **kwargs):
+        """Generate resource."""
+        return Database(**data)
+
+
 class SlurmSettingsSchema(BaseSchema):
     """Represent the schema of the Scheduling Settings."""
 
@@ -1237,6 +1373,7 @@ class SlurmSettingsSchema(BaseSchema):
         metadata={"update_policy": UpdatePolicy.IGNORED},
     )
     enable_memory_based_scheduling = fields.Bool(metadata={"update_policy": UpdatePolicy.COMPUTE_FLEET_STOP})
+    database = fields.Nested(DatabaseSchema, metadata={"update_policy": UpdatePolicy.COMPUTE_FLEET_STOP})
 
     @post_load
     def make_resource(self, data, **kwargs):
@@ -1635,7 +1772,7 @@ class SchedulerPluginSettingsSchema(BaseSchema):
 
         LOGGER.info("Using the following scheduler plugin definition:\n%s", scheduler_definition)
         try:
-            data["SchedulerDefinition"] = yaml.safe_load(scheduler_definition)
+            data["SchedulerDefinition"] = yaml_load(scheduler_definition)
         except YAMLError as e:
             raise ValidationError(
                 f"The retrieved SchedulerDefinition ({original_scheduler_definition}) is not a valid YAML."
@@ -1736,19 +1873,6 @@ class SchedulingSchema(BaseSchema):
                     f"Scheduling > *Queues section is not appropriate to the Scheduler: {configured_scheduler}."
                 )
 
-    @validates_schema
-    def same_subnet_in_different_queues(self, data, **kwargs):
-        """Validate subnet_ids configured in different queues are the same."""
-        if "slurm_queues" in data or "scheduler_queues" in data:
-            queues = "slurm_queues" if "slurm_queues" in data else "scheduler_queues"
-
-            def _queue_has_subnet_ids(queue):
-                return queue.networking and queue.networking.subnet_ids
-
-            subnet_ids = {tuple(set(q.networking.subnet_ids)) for q in data[queues] if _queue_has_subnet_ids(q)}
-            if len(subnet_ids) > 1:
-                raise ValidationError("The SubnetIds used for all of the queues should be the same.")
-
     @post_load
     def make_resource(self, data, **kwargs):
         """Generate the right type of scheduling according to the child type (Slurm vs AwsBatch vs Custom)."""
@@ -1816,9 +1940,7 @@ class ClusterSchema(BaseSchema):
         SharedStorageSchema,
         many=True,
         metadata={
-            "update_policy": UpdatePolicy(
-                UpdatePolicy.UNSUPPORTED, fail_reason=UpdatePolicy.FAIL_REASONS["shared_storage_change"]
-            ),
+            "update_policy": UpdatePolicy(UpdatePolicy.SHARED_STORAGE_UPDATE_POLICY),
             "update_key": "Name",
         },
     )
@@ -1833,6 +1955,7 @@ class ClusterSchema(BaseSchema):
         DirectoryServiceSchema, metadata={"update_policy": UpdatePolicy.COMPUTE_FLEET_STOP}
     )
     config_region = fields.Str(data_key="Region", metadata={"update_policy": UpdatePolicy.UNSUPPORTED})
+    imds = fields.Nested(TopLevelImdsSchema, metadata={"update_policy": UpdatePolicy.UNSUPPORTED})
     custom_s3_bucket = fields.Str(metadata={"update_policy": UpdatePolicy.READ_ONLY_RESOURCE_BUCKET})
     additional_resources = fields.Str(metadata={"update_policy": UpdatePolicy.SUPPORTED})
     dev_settings = fields.Nested(ClusterDevSettingsSchema, metadata={"update_policy": UpdatePolicy.SUPPORTED})
