@@ -20,7 +20,7 @@ import json
 from marshmallow import Schema, ValidationError, fields, post_dump, post_load, pre_dump, validate, validates
 
 from pcluster.config.cluster_config import BaseTag
-from pcluster.config.common import AdditionalIamPolicy, Cookbook
+from pcluster.config.common import AdditionalIamPolicy, Cookbook, Imds
 from pcluster.config.update_policy import UpdatePolicy
 from pcluster.constants import PCLUSTER_PREFIX, SUPPORTED_ARCHITECTURES
 from pcluster.utils import to_pascal_case
@@ -36,6 +36,8 @@ ALLOWED_VALUES = {
     "volume_type": ["standard", "io1", "io2", "gp2", "st1", "sc1", "gp3"],
 }
 
+SUPPORTED_IMDS_VERSIONS = ["v1.0", "v2.0"]
+
 
 def validate_json_format(data):
     """Validate the input data in json format."""
@@ -64,7 +66,7 @@ class BaseSchema(Schema):
 
     def on_bind_field(self, field_name, field_obj):
         """
-        Bind PascalCase in the config with with snake_case in Python.
+        Bind PascalCase in the config with snake_case in Python.
 
         For example, subnet_id in the code is automatically bind with SubnetId in the config file.
         The bind can be overwritten by specifying data_key.
@@ -76,11 +78,11 @@ class BaseSchema(Schema):
     @staticmethod
     def fields_coexist(data, field_list, one_required=False, **kwargs):
         """
-        Check if at least two fields in the filed lists co-exist in the schema.
+        Check if at least two fields in the field list co-exist in the schema.
 
         :param data: data to be checked
         :param field_list: list including the name of the fields to check
-        :param one_required: True if one of the field is required to be existed
+        :param one_required: True if one of the fields is required to exist
         :return: True if one and only one field is not None
         """
         if kwargs.get("partial"):
@@ -137,10 +139,16 @@ class TagSchema(BaseSchema):
     """Represent the schema of Tag section."""
 
     key = fields.Str(
-        required=True, validate=validate.Length(max=128), metadata={"update_policy": UpdatePolicy.UNSUPPORTED}
+        # TODO Tags can be updated with policy QUEUE_UPDATE_STRATEGY
+        required=True,
+        validate=validate.Length(max=128),
+        metadata={"update_policy": UpdatePolicy.UNSUPPORTED},
     )
     value = fields.Str(
-        required=True, validate=validate.Length(max=256), metadata={"update_policy": UpdatePolicy.UNSUPPORTED}
+        # TODO Tags can be updated with policy QUEUE_UPDATE_STRATEGY
+        required=True,
+        validate=validate.Length(max=256),
+        metadata={"update_policy": UpdatePolicy.UNSUPPORTED},
     )
 
     @post_load
@@ -199,3 +207,23 @@ class BaseDevSettingsSchema(BaseSchema):
     cookbook = fields.Nested(CookbookSchema, metadata={"update_policy": UpdatePolicy.UNSUPPORTED})
     node_package = fields.Str(metadata={"update_policy": UpdatePolicy.UNSUPPORTED})
     aws_batch_cli_package = fields.Str(metadata={"update_policy": UpdatePolicy.UNSUPPORTED})
+
+
+class ImdsSchema(BaseSchema):
+    """
+    Represent the Imds schema shared between cluster and build image files.
+
+    It represents the Imds element that can be either at top level in the cluster config file,
+    or in the Build section of the build image config file.
+    """
+
+    imds_support = fields.Str(
+        data_key="ImdsSupport",
+        validate=validate.OneOf(SUPPORTED_IMDS_VERSIONS),
+        metadata={"update_policy": UpdatePolicy.UNSUPPORTED},
+    )
+
+    @post_load
+    def make_resource(self, data, **kwargs):
+        """Generate resource."""
+        return Imds(**data)
