@@ -491,7 +491,7 @@ class HeadNodeIamResources(NodeIamResourcesBase):
             ),
             iam.PolicyStatement(
                 sid="Ec2TagsAndVolumes",
-                actions=["ec2:AttachVolume", "ec2:CreateTags"],
+                actions=["ec2:AttachVolume", "ec2:CreateTags", "ec2:DetachVolume"],
                 effect=iam.Effect.ALLOW,
                 resources=[
                     self._format_arn(
@@ -578,14 +578,15 @@ class HeadNodeIamResources(NodeIamResourcesBase):
                         },
                     ),
                     iam.PolicyStatement(
-                        sid="EC2RunInstances",
-                        actions=["ec2:RunInstances"],
+                        sid="EC2RunInstancesCreateFleet",
+                        actions=["ec2:RunInstances", "ec2:CreateFleet"],
                         effect=iam.Effect.ALLOW,
                         resources=[
                             self._format_arn(service="ec2", resource=f"subnet/{subnet_id}")
                             for subnet_id in self._config.compute_subnet_ids
                         ]
                         + [
+                            self._format_arn(service="ec2", resource="fleet/*"),
                             self._format_arn(service="ec2", resource="network-interface/*"),
                             self._format_arn(service="ec2", resource="instance/*"),
                             self._format_arn(service="ec2", resource="volume/*"),
@@ -599,6 +600,12 @@ class HeadNodeIamResources(NodeIamResourcesBase):
                             for _, queue_ami in self._config.image_dict.items()
                         ],
                     ),
+                    iam.PolicyStatement(
+                        sid="EC2DescribeCapacityReservations",
+                        actions=["ec2:DescribeCapacityReservations"],
+                        effect=iam.Effect.ALLOW,
+                        resources=["*"],
+                    ),
                     iam.PolicyStatement(
                         sid="PassRole",
                         actions=["iam:PassRole"],
@@ -618,6 +625,33 @@ class HeadNodeIamResources(NodeIamResourcesBase):
                     ),
                 ]
             )
+            capacity_reservation_ids = self._config.capacity_reservation_ids
+
+            if capacity_reservation_ids:
+                policy.append(
+                    iam.PolicyStatement(
+                        actions=["ec2:RunInstances"],
+                        effect=iam.Effect.ALLOW,
+                        resources=[
+                            self._format_arn(
+                                service="ec2",
+                                resource=f"capacity-reservation/{capacity_reservation_id}",
+                            )
+                            for capacity_reservation_id in capacity_reservation_ids
+                        ],
+                    )
+                )
+            capacity_reservation_resource_group_arns = self._config.capacity_reservation_resource_group_arns
+            if capacity_reservation_resource_group_arns:
+                policy.extend(
+                    [
+                        iam.PolicyStatement(
+                            actions=["ec2:RunInstances", "ec2:CreateFleet", "resource-groups:ListGroupResources"],
+                            effect=iam.Effect.ALLOW,
+                            resources=capacity_reservation_resource_group_arns,
+                        )
+                    ]
+                )
 
         if self._config.scheduling.scheduler == "plugin":
             cluster_shared_artifacts = get_attr(
@@ -656,6 +690,15 @@ class HeadNodeIamResources(NodeIamResourcesBase):
                 )
             )
 
+        if self._config.scheduling.scheduler == "slurm" and self._config.scheduling.settings.database:
+            policy.append(
+                iam.PolicyStatement(
+                    actions=["secretsmanager:GetSecretValue"],
+                    effect=iam.Effect.ALLOW,
+                    resources=[self._config.scheduling.settings.database.password_secret_arn],
+                )
+            )
+
         return policy
 
     def _generate_head_node_pass_role_resources(self):
@@ -762,7 +805,7 @@ class PclusterLambdaConstruct(Construct):
             handler=f"{handler_func}.handler",
             memory_size=128,
             role=execution_role,
-            runtime="python3.8",
+            runtime="python3.9",
             timeout=timeout,
         )
 
