@@ -54,6 +54,7 @@ from pcluster.config.cluster_config import (
     SharedFsxLustre,
     SharedStorageType,
     SlurmClusterConfig,
+    SlurmComputeResource,
     SlurmQueue,
 )
 from pcluster.constants import (
@@ -98,7 +99,7 @@ from pcluster.templates.cdk_builder_utils import (
 )
 from pcluster.templates.cw_dashboard_builder import CWDashboardConstruct
 from pcluster.templates.slurm_builder import SlurmConstruct
-from pcluster.utils import get_attr, join_shell_args
+from pcluster.utils import get_attr, get_http_tokens_setting, join_shell_args
 
 StorageInfo = namedtuple("StorageInfo", ["id", "config"])
 
@@ -515,7 +516,7 @@ class ClusterCdkStack(Stack):
                 )
 
     def _allow_all_ingress(self, description, source_security_group_id, group_id):
-        ec2.CfnSecurityGroupIngress(
+        return ec2.CfnSecurityGroupIngress(
             self,
             description,
             ip_protocol="-1",
@@ -525,6 +526,60 @@ class ClusterCdkStack(Stack):
             group_id=group_id,
         )
 
+    def _allow_all_egress(self, description, destination_security_group_id, group_id):
+        return ec2.CfnSecurityGroupEgress(
+            self,
+            description,
+            ip_protocol="-1",
+            from_port=0,
+            to_port=65535,
+            destination_security_group_id=destination_security_group_id,
+            group_id=group_id,
+        )
+
+    def _add_storage_security_group(self, storage_cfn_id, storage):
+        storage_type = storage.shared_storage_type
+        storage_security_group = ec2.CfnSecurityGroup(
+            self,
+            "{0}SecurityGroup".format(storage_cfn_id),
+            group_description=f"Allow access to {storage_type} file system {storage_cfn_id}",
+            vpc_id=self.config.vpc_id,
+        )
+        storage_deletion_policy = convert_deletion_policy(storage.deletion_policy)
+        storage_security_group.cfn_options.deletion_policy = (
+            storage_security_group.cfn_options.update_replace_policy
+        ) = storage_deletion_policy
+
+        target_security_groups = {
+            "Head": self._get_head_node_security_groups(),
+            "Compute": self._get_compute_security_groups(),
+            "Storage": [storage_security_group.ref],
+        }
+
+        for sg_type, sg_refs in target_security_groups.items():
+            for sg_ref_id, sg_ref in enumerate(sg_refs):
+                ingress_rule = self._allow_all_ingress(
+                    description=f"{storage_cfn_id}SecurityGroup{sg_type}Ingress{sg_ref_id}",
+                    source_security_group_id=sg_ref,
+                    group_id=storage_security_group.ref,
+                )
+
+                egress_rule = self._allow_all_egress(
+                    description=f"{storage_cfn_id}SecurityGroup{sg_type}Egress{sg_ref_id}",
+                    destination_security_group_id=sg_ref,
+                    group_id=storage_security_group.ref,
+                )
+
+                if sg_type == "Storage":
+                    ingress_rule.cfn_options.deletion_policy = (
+                        ingress_rule.cfn_options.update_replace_policy
+                    ) = storage_deletion_policy
+                    egress_rule.cfn_options.deletion_policy = (
+                        egress_rule.cfn_options.update_replace_policy
+                    ) = storage_deletion_policy
+
+        return storage_security_group
+
     def _add_compute_security_group(self):
         compute_security_group = ec2.CfnSecurityGroup(
             self, "ComputeSecurityGroup", group_description="Allow access to compute nodes", vpc_id=self.config.vpc_id
@@ -597,7 +652,7 @@ class ClusterCdkStack(Stack):
     def _add_shared_storage(self, storage):
         """Add specific Cfn Resources to map the shared storage and store the output filesystem id."""
         storage_list = self.shared_storage_infos[storage.shared_storage_type]
-        cfn_resource_id = "{0}{1}".format(storage.shared_storage_type.name, len(storage_list))
+        cfn_resource_id = "{0}{1}".format(storage.shared_storage_type.name, create_hash_suffix(storage.name))
         if storage.shared_storage_type == SharedStorageType.FSX:
             storage_list.append(StorageInfo(self._add_fsx_storage(cfn_resource_id, storage), storage))
         elif storage.shared_storage_type == SharedStorageType.EBS:
@@ -630,6 +685,7 @@ class ClusterCdkStack(Stack):
                     drive_cache_type = shared_fsx.drive_cache_type
                 else:
                     drive_cache_type = "NONE"
+            file_system_security_groups = [self._add_storage_security_group(id, shared_fsx)]
             fsx_resource = fsx.CfnFileSystem(
                 self,
                 id,
@@ -653,10 +709,14 @@ class ClusterCdkStack(Stack):
                 file_system_type=LUSTRE,
                 storage_type=shared_fsx.fsx_storage_type,
                 subnet_ids=self.config.compute_subnet_ids,
-                security_group_ids=self._get_compute_security_groups(),
+                security_group_ids=[sg.ref for sg in file_system_security_groups],
                 file_system_type_version=shared_fsx.file_system_type_version,
                 tags=[CfnTag(key="Name", value=shared_fsx.name)],
             )
+            fsx_resource.cfn_options.deletion_policy = (
+                fsx_resource.cfn_options.update_replace_policy
+            ) = convert_deletion_policy(shared_fsx.deletion_policy)
+
             fsx_id = fsx_resource.ref
             # Get MountName for new filesystem. DNSName cannot be retrieved from CFN and will be generated in cookbook
             mount_name = fsx_resource.attr_lustre_mount_name
@@ -677,6 +737,7 @@ class ClusterCdkStack(Stack):
         # EFS FileSystem
         efs_id = shared_efs.file_system_id
         new_file_system = efs_id is None
+        deletion_policy = convert_deletion_policy(shared_efs.deletion_policy)
         if not efs_id and shared_efs.mount_dir:
             efs_resource = efs.CfnFileSystem(
                 self,
@@ -688,27 +749,35 @@ class ClusterCdkStack(Stack):
                 throughput_mode=shared_efs.throughput_mode,
             )
             efs_resource.tags.set_tag(key="Name", value=shared_efs.name)
+            efs_resource.cfn_options.deletion_policy = efs_resource.cfn_options.update_replace_policy = deletion_policy
             efs_id = efs_resource.ref
 
         checked_availability_zones = []
 
         # Mount Targets for Compute Fleet
         compute_subnet_ids = self.config.compute_subnet_ids
-        compute_node_sgs = self._get_compute_security_groups()
+        file_system_security_groups = [self._add_storage_security_group(id, shared_efs)]
 
         for subnet_id in compute_subnet_ids:
             self._add_efs_mount_target(
-                id, efs_id, compute_node_sgs, subnet_id, checked_availability_zones, new_file_system
+                id,
+                efs_id,
+                file_system_security_groups,
+                subnet_id,
+                checked_availability_zones,
+                new_file_system,
+                deletion_policy,
             )
 
         # Mount Target for Head Node
         self._add_efs_mount_target(
             id,
             efs_id,
-            compute_node_sgs,
+            file_system_security_groups,
             self.config.head_node.networking.subnet_id,
             checked_availability_zones,
             new_file_system,
+            deletion_policy,
         )
 
         return efs_id
@@ -721,18 +790,22 @@ class ClusterCdkStack(Stack):
         subnet_id,
         checked_availability_zones,
         new_file_system,
+        deletion_policy,
     ):
         """Create a EFS Mount Point for the file system, if not already available on the same AZ."""
         availability_zone = AWSApi.instance().ec2.get_subnet_avail_zone(subnet_id)
         if availability_zone not in checked_availability_zones:
             if new_file_system or not AWSApi.instance().efs.get_efs_mount_target_id(file_system_id, availability_zone):
-                efs.CfnMountTarget(
+                efs_resource = efs.CfnMountTarget(
                     self,
                     "{0}MT{1}".format(efs_cfn_resource_id, availability_zone),
                     file_system_id=file_system_id,
-                    security_groups=security_groups,
+                    security_groups=[sg.ref for sg in security_groups],
                     subnet_id=subnet_id,
                 )
+                efs_resource.cfn_options.deletion_policy = (
+                    efs_resource.cfn_options.update_replace_policy
+                ) = deletion_policy
             checked_availability_zones.append(availability_zone)
 
     def _add_raid_volume(self, id_prefix: str, shared_ebs: SharedEbs):
@@ -766,7 +839,9 @@ class ClusterCdkStack(Stack):
             volume_type=shared_ebs.volume_type,
             tags=[CfnTag(key="Name", value=shared_ebs.name)],
         )
-        volume.cfn_options.deletion_policy = convert_deletion_policy(shared_ebs.deletion_policy)
+        volume.cfn_options.deletion_policy = volume.cfn_options.update_replace_policy = convert_deletion_policy(
+            shared_ebs.deletion_policy
+        )
         return volume.ref
 
     def _add_wait_condition(self):
@@ -795,11 +870,11 @@ class ClusterCdkStack(Stack):
                 network_interface_id=self._head_eni.ref,
             )
         ]
-        for device_index in range(1, head_node.max_network_interface_count):
+        for network_interface_index in range(1, head_node.max_network_interface_count):
             head_lt_nw_interfaces.append(
                 ec2.CfnLaunchTemplate.NetworkInterfaceProperty(
-                    device_index=device_index,
-                    network_card_index=device_index,
+                    device_index=1,
+                    network_card_index=network_interface_index,
                     groups=head_lt_security_groups,
                     subnet_id=head_node.networking.subnet_id,
                 )
@@ -811,9 +886,6 @@ class ClusterCdkStack(Stack):
             "HeadNodeLaunchTemplate",
             launch_template_data=ec2.CfnLaunchTemplate.LaunchTemplateDataProperty(
                 instance_type=head_node.instance_type,
-                cpu_options=ec2.CfnLaunchTemplate.CpuOptionsProperty(core_count=head_node.vcpus, threads_per_core=1)
-                if head_node.pass_cpu_options_in_launch_template
-                else None,
                 block_device_mappings=get_block_device_mappings(head_node.local_storage, self.config.image.os),
                 key_name=head_node.ssh.key_name,
                 network_interfaces=head_lt_nw_interfaces,
@@ -822,20 +894,23 @@ class ClusterCdkStack(Stack):
                 iam_instance_profile=ec2.CfnLaunchTemplate.IamInstanceProfileProperty(
                     name=self._head_node_instance_profile
                 ),
+                metadata_options=ec2.CfnLaunchTemplate.MetadataOptionsProperty(
+                    http_tokens=get_http_tokens_setting(self.config.imds.imds_support)
+                ),
                 user_data=Fn.base64(
                     Fn.sub(
                         get_user_data_content("../resources/head_node/user_data.sh"),
-                        {**get_common_user_data_env(head_node, self.config)},
+                        {
+                            **{
+                                "DisableMultiThreadingManually": "true"
+                                if head_node.disable_simultaneous_multithreading_manually
+                                else "false",
+                            },
+                            **get_common_user_data_env(head_node, self.config),
+                        },
                     )
                 ),
                 tag_specifications=[
-                    ec2.CfnLaunchTemplate.TagSpecificationProperty(
-                        resource_type="instance",
-                        tags=get_default_instance_tags(
-                            self._stack_name, self.config, head_node, "HeadNode", self.shared_storage_infos
-                        )
-                        + get_custom_tags(self.config),
-                    ),
                     ec2.CfnLaunchTemplate.TagSpecificationProperty(
                         resource_type="volume",
                         tags=get_default_volume_tags(self._stack_name, "HeadNode") + get_custom_tags(self.config),
@@ -867,9 +942,6 @@ class ClusterCdkStack(Stack):
                     "raid_type": to_comma_separated_string(
                         self.shared_storage_attributes[SharedStorageType.RAID]["Type"]
                     ),
-                    "disable_hyperthreading_manually": "true"
-                    if head_node.disable_simultaneous_multithreading_manually
-                    else "false",
                     "base_os": self.config.image.os,
                     "preinstall": pre_install_action.script if pre_install_action else "NONE",
                     "preinstall_args": join_shell_args(pre_install_action.args)
@@ -1110,6 +1182,10 @@ class ClusterCdkStack(Stack):
                 launch_template_id=head_node_launch_template.ref,
                 version=head_node_launch_template.attr_latest_version_number,
             ),
+            tags=get_default_instance_tags(
+                self._stack_name, self.config, head_node, "HeadNode", self.shared_storage_infos
+            )
+            + get_custom_tags(self.config),
         )
         if not self._condition_is_batch():
             head_node_instance.node.add_dependency(self.compute_fleet_resources)
@@ -1308,47 +1384,41 @@ class ComputeFleetConstruct(Construct):
     def _add_placement_groups(self) -> Dict[str, ec2.CfnPlacementGroup]:
         managed_placement_groups = {}
         for queue in self._config.scheduling.queues:
-            if (
-                queue.networking.placement_group
-                and queue.networking.placement_group.enabled
-                and not queue.networking.placement_group.id
-            ):
-                managed_placement_groups[queue.name] = ec2.CfnPlacementGroup(
-                    self, f"PlacementGroup{create_hash_suffix(queue.name)}", strategy="cluster"
+            for key in queue.get_managed_placement_group_keys():
+                managed_placement_groups[key] = ec2.CfnPlacementGroup(
+                    self,
+                    f"PlacementGroup{create_hash_suffix(key)}",
+                    strategy="cluster",
                 )
         return managed_placement_groups
 
+    @staticmethod
+    def _get_placement_group_for_compute_resource(queue, managed_placement_groups, compute_resource) -> str:
+        placement_group_settings = queue.get_placement_group_settings_for_compute_resource(compute_resource)
+        placement_group_key = placement_group_settings.get("key")
+        managed = placement_group_settings.get("is_managed")
+        return managed_placement_groups[placement_group_key].ref if managed else placement_group_key
+
     def _add_launch_templates(self, managed_placement_groups, instance_profiles):
         compute_launch_templates = {}
         for queue in self._config.scheduling.queues:
             compute_launch_templates[queue.name] = {}
             queue_lt_security_groups = get_queue_security_groups_full(self._compute_security_group, queue)
-
-            queue_placement_group = None
-            if queue.networking.placement_group:
-                if queue.networking.placement_group.id and (
-                    queue.networking.placement_group.enabled or queue.networking.placement_group.is_implied("enabled")
-                ):  # Do not use `PlacementGroup/Id` when `PlacementGroup/Enabled` is explicitly set to `false`
-                    queue_placement_group = queue.networking.placement_group.id
-                elif queue.networking.placement_group.enabled:
-                    queue_placement_group = managed_placement_groups[queue.name].ref
-
             queue_pre_install_action, queue_post_install_action = (None, None)
             if queue.custom_actions:
                 queue_pre_install_action = queue.custom_actions.on_node_start
                 queue_post_install_action = queue.custom_actions.on_node_configured
 
-            for compute_resource in queue.compute_resources:
-                launch_template = self._add_compute_resource_launch_template(
+            for resource in queue.compute_resources:
+                compute_launch_templates[queue.name][resource.name] = self._add_compute_resource_launch_template(
                     queue,
-                    compute_resource,
+                    resource,
                     queue_pre_install_action,
                     queue_post_install_action,
                     queue_lt_security_groups,
-                    queue_placement_group,
+                    self._get_placement_group_for_compute_resource(queue, managed_placement_groups, resource),
                     instance_profiles,
                 )
-                compute_launch_templates[queue.name][compute_resource.name] = launch_template
         return compute_launch_templates
 
     def _add_compute_resource_launch_template(
@@ -1358,7 +1428,7 @@ class ComputeFleetConstruct(Construct):
         queue_pre_install_action,
         queue_post_install_action,
         queue_lt_security_groups,
-        queue_placement_group,
+        placement_group,
         instance_profiles,
     ):
         # LT network interfaces
@@ -1373,10 +1443,11 @@ class ComputeFleetConstruct(Construct):
                 subnet_id=queue.networking.subnet_ids[0],
             )
         ]
+
         for network_interface_index in range(1, compute_resource.max_network_interface_count):
             compute_lt_nw_interfaces.append(
                 ec2.CfnLaunchTemplate.NetworkInterfaceProperty(
-                    device_index=1,
+                    device_index=0,
                     network_card_index=network_interface_index,
                     interface_type="efa" if compute_resource.efa and compute_resource.efa.enabled else None,
                     groups=queue_lt_security_groups,
@@ -1395,30 +1466,44 @@ class ComputeFleetConstruct(Construct):
                 ),
             )
 
+        conditional_template_properties = {}
+
+        if compute_resource.is_ebs_optimized:
+            conditional_template_properties.update({"ebs_optimized": True})
+        if isinstance(compute_resource, SlurmComputeResource):
+            conditional_template_properties.update({"instance_type": compute_resource.instance_type})
+
+        capacity_reservation_specification = None
+        cr_target = compute_resource.capacity_reservation_target or queue.capacity_reservation_target
+        if cr_target:
+            capacity_reservation_specification = ec2.CfnLaunchTemplate.CapacityReservationSpecificationProperty(
+                capacity_reservation_target=ec2.CfnLaunchTemplate.CapacityReservationTargetProperty(
+                    capacity_reservation_id=cr_target.capacity_reservation_id,
+                    capacity_reservation_resource_group_arn=cr_target.capacity_reservation_resource_group_arn,
+                )
+            )
+
         return ec2.CfnLaunchTemplate(
             self,
             f"LaunchTemplate{create_hash_suffix(queue.name + compute_resource.name)}",
             launch_template_name=f"{self.stack_name}-{queue.name}-{compute_resource.name}",
             launch_template_data=ec2.CfnLaunchTemplate.LaunchTemplateDataProperty(
-                instance_type=compute_resource.instance_type,
-                cpu_options=ec2.CfnLaunchTemplate.CpuOptionsProperty(
-                    core_count=compute_resource.vcpus, threads_per_core=1
-                )
-                if compute_resource.pass_cpu_options_in_launch_template
-                else None,
                 block_device_mappings=get_block_device_mappings(
                     queue.compute_settings.local_storage, self._config.image.os
                 ),
                 # key_name=,
                 network_interfaces=compute_lt_nw_interfaces,
-                placement=ec2.CfnLaunchTemplate.PlacementProperty(group_name=queue_placement_group),
+                placement=ec2.CfnLaunchTemplate.PlacementProperty(group_name=placement_group),
                 image_id=self._config.image_dict[queue.name],
-                ebs_optimized=compute_resource.is_ebs_optimized,
                 iam_instance_profile=ec2.CfnLaunchTemplate.IamInstanceProfileProperty(
                     name=instance_profiles[queue.name]
                 ),
                 instance_market_options=instance_market_options,
                 instance_initiated_shutdown_behavior="terminate",
+                capacity_reservation_specification=capacity_reservation_specification,
+                metadata_options=ec2.CfnLaunchTemplate.MetadataOptionsProperty(
+                    http_tokens=get_http_tokens_setting(self._config.imds.imds_support)
+                ),
                 user_data=Fn.base64(
                     Fn.sub(
                         get_user_data_content("../resources/compute_node/user_data.sh"),
@@ -1431,7 +1516,7 @@ class ComputeFleetConstruct(Construct):
                                 "RAIDType": to_comma_separated_string(
                                     self._shared_storage_attributes[SharedStorageType.RAID]["Type"]
                                 ),
-                                "DisableHyperThreadingManually": "true"
+                                "DisableMultiThreadingManually": "true"
                                 if compute_resource.disable_simultaneous_multithreading_manually
                                 else "false",
                                 "BaseOS": self._config.image.os,
@@ -1536,5 +1621,6 @@ class ComputeFleetConstruct(Construct):
                         + get_custom_tags(self._config),
                     ),
                 ],
+                **conditional_template_properties,
             ),
         )
