@@ -52,6 +52,7 @@ from pcluster.imagebuilder_utils import (
 )
 from pcluster.models.s3_bucket import S3Bucket, S3FileType, create_s3_presigned_url, parse_bucket_url
 from pcluster.templates.cdk_builder_utils import apply_permissions_boundary, get_assume_role_policy_document
+from pcluster.utils import get_http_tokens_setting
 
 
 class ImageBuilderCdkStack(Stack):
@@ -583,6 +584,9 @@ class ImageBuilderCdkStack(Stack):
             security_group_ids=self.config.build.security_group_ids,
             subnet_id=self.config.build.subnet_id,
             sns_topic_arn=Fn.ref("BuildNotificationTopic"),
+            instance_metadata_options=imagebuilder.CfnInfrastructureConfiguration.InstanceMetadataOptionsProperty(
+                http_tokens=get_http_tokens_setting(self.config.build.imds.imds_support)
+            ),
         )
         if not self.custom_cleanup_lambda_role:
             self._add_resource_delete_policy(
@@ -774,7 +778,7 @@ class ImageBuilderCdkStack(Stack):
             handler="delete_image_stack.handler",
             memory_size=128,
             role=execution_role,
-            runtime="python3.8",
+            runtime="python3.9",
             timeout=900,
             environment=lambda_env,
             tags=build_tags,
