@@ -12,19 +12,24 @@ import math
 import re
 from collections import defaultdict
 from enum import Enum
+from ipaddress import ip_network
 from itertools import combinations, product
+from typing import List
 
 from pcluster.aws.aws_api import AWSApi
+from pcluster.aws.aws_resources import InstanceTypeInfo
 from pcluster.aws.common import AWSClientError
 from pcluster.cli.commands.dcv_util import get_supported_dcv_os
 from pcluster.constants import (
     CIDR_ALL_IPS,
+    DELETE_POLICY,
     FSX_PORTS,
     PCLUSTER_IMAGE_BUILD_STATUS_TAG,
     PCLUSTER_NAME_MAX_LENGTH,
     PCLUSTER_NAME_REGEX,
     PCLUSTER_TAG_VALUE_REGEX,
     PCLUSTER_VERSION_TAG,
+    RETAIN_POLICY,
     SCHEDULERS_SUPPORTING_IMDS_SECURED,
     SUPPORTED_OSES,
     SUPPORTED_REGIONS,
@@ -40,6 +45,12 @@ NAME_REGEX = r"^[a-z][a-z0-9\-]*$"
 
 EFA_UNSUPPORTED_ARCHITECTURES_OSES = {"x86_64": [], "arm64": ["centos7"]}
 
+EFS_MESSAGES = {
+    "errors": {
+        "ignored_param_with_efs_fs_id": "{efs_param} cannot be specified when an existing EFS file system is used.",
+    }
+}
+
 FSX_SUPPORTED_ARCHITECTURES_OSES = {
     "x86_64": SUPPORTED_OSES,
     "arm64": ["ubuntu1804", "ubuntu2004", "alinux2", "centos7"],
@@ -54,7 +65,7 @@ FSX_MESSAGES = {
         "compute instance type and/or custom AMI configurations.",
         "unsupported_backup_param": "When restoring an FSx Lustre file system from backup, '{name}' "
         "cannot be specified.",
-        "ignored_param_with_fsx_fs_id": "{fsx_param} is ignored when an existing Lustre file system is specified.",
+        "ignored_param_with_fsx_fs_id": "{fsx_param} cannot be specified when an existing Lustre file system is used.",
     }
 }
 
@@ -227,17 +238,19 @@ class InstanceArchitectureCompatibilityValidator(Validator):
     Verify that head node and compute instance types imply compatible architectures.
     """
 
-    def _validate(self, instance_type, architecture: str):
+    def _validate(self, instance_type_info_list: List[InstanceTypeInfo], architecture: str):
         head_node_architecture = architecture
-        compute_architectures = AWSApi.instance().ec2.get_supported_architectures(instance_type)
-        if head_node_architecture not in compute_architectures:
-            self._add_failure(
-                "The specified compute instance type ({0}) supports the architectures {1}, none of which are "
-                "compatible with the architecture supported by the head node instance type ({2}).".format(
-                    instance_type, compute_architectures, head_node_architecture
-                ),
-                FailureLevel.ERROR,
-            )
+
+        for instance_type_info in instance_type_info_list:
+            compute_architectures = instance_type_info.supported_architecture()
+            if head_node_architecture not in instance_type_info.supported_architecture():
+                self._add_failure(
+                    "The specified compute instance type ({0}) supports the architectures {1}, none of which are "
+                    "compatible with the architecture supported by the head node instance type ({2}).".format(
+                        instance_type_info.instance_type(), compute_architectures, head_node_architecture
+                    ),
+                    FailureLevel.ERROR,
+                )
 
 
 class NameValidator(Validator):
@@ -304,24 +317,18 @@ class EfaValidator(Validator):
 class EfaPlacementGroupValidator(Validator):
     """Validate placement group if EFA is enabled."""
 
-    def _validate(self, efa_enabled, placement_group):
-        placement_group_enabled = placement_group and (
-            placement_group.enabled or (placement_group.id and placement_group.is_implied("enabled"))
-        )
-        placement_group_config_implicit = placement_group is None or (
-            placement_group.is_implied("enabled") and placement_group.id is None
-        )
-        if efa_enabled and placement_group_config_implicit:
+    def _validate(self, efa_enabled: bool, placement_group_key: str, placement_group_disabled: bool):
+        if efa_enabled and placement_group_disabled:
+            self._add_failure(
+                "You may see better performance using a placement group for the queue.", FailureLevel.WARNING
+            )
+        elif efa_enabled and placement_group_key is None:
             self._add_failure(
                 "The placement group for EFA-enabled compute resources must be explicit. "
                 "You may see better performance using a placement group, but if you don't wish to use one please add "
                 "'Enabled: false' to the compute resource's configuration section.",
                 FailureLevel.ERROR,
             )
-        elif efa_enabled and not placement_group_enabled:
-            self._add_failure(
-                "You may see better performance using a placement group for the queue.", FailureLevel.WARNING
-            )
 
 
 class EfaSecurityGroupValidator(Validator):
@@ -658,8 +665,8 @@ class EfsIdValidator(Validator):  # TODO add tests
     Validate if there are existing mount target in the head node availability zone
     """
 
-    def _validate(self, efs_id, avail_zones: set, are_all_security_groups_customized):
-        for avail_zone in avail_zones:
+    def _validate(self, efs_id, avail_zones_mapping: dict, are_all_security_groups_customized):
+        for avail_zone, subnets in avail_zones_mapping.items():
             head_node_target_id = AWSApi.instance().efs.get_efs_mount_target_id(efs_id, avail_zone)
             # If there is an existing mt in the az, need to check the inbound and outbound rules of the security groups
             if head_node_target_id:
@@ -674,6 +681,44 @@ class EfsIdValidator(Validator):  # TODO add tests
                         ),
                         FailureLevel.ERROR,
                     )
+                if not are_all_security_groups_customized:
+                    self._check_cidrs_cover_subnets(head_node_target_id, avail_zone, sg_ids, efs_id, subnets)
+
+    def _check_subnet_access(self, security_groups_ids, subnet_cidr, access_type):
+        permission = "IpPermissions" if access_type == "in" else "IpPermissionsEgress"
+        access = False
+        for sec_group in security_groups_ids:
+            for rule in sec_group.get(permission):
+                if rule.get("PrefixListIds"):
+                    access = True
+                    break
+                if rule.get("IpRanges"):
+                    for ip_range in rule.get("IpRanges"):
+                        if ip_network(ip_range.get("CidrIp")).supernet_of(subnet_cidr):
+                            access = True
+                            break
+        return access
+
+    def _check_cidrs_cover_subnets(self, head_node_target_id, avail_zone, security_groups_ids, efs_id, subnets):
+        """Verify given list of security groups to check if they allow in and out access on cluster subnet CIDRs."""
+        security_groups_ids = AWSApi.instance().ec2.describe_security_groups(security_groups_ids)
+        for subnet in subnets:
+            subnet_cidr = ip_network(AWSApi.instance().ec2.get_subnet_cidr(subnet))
+            in_access, out_access = self._check_subnet_access(
+                security_groups_ids, subnet_cidr, "in"
+            ), self._check_subnet_access(security_groups_ids, subnet_cidr, "out")
+
+            if not in_access or not out_access:
+                self._add_failure(
+                    "There is an existing Mount Target {0} in the Availability Zone {1} for EFS {2}, "
+                    "but it does not have a security group that allows inbound and outbound rules to allow traffic of "
+                    "subnet {3}. Please modify the Mount Target's security group, to allow traffic on subnet.".format(
+                        head_node_target_id, avail_zone, efs_id, subnet
+                    ),
+                    FailureLevel.WARNING,
+                )
+
+        return False
 
 
 class SharedStorageNameValidator(Validator):
@@ -743,6 +788,24 @@ class SharedStorageMountDirValidator(Validator):
             )
 
 
+class DeletionPolicyValidator(Validator):
+    """Print warning message when deletion policy is set to Delete or Retain."""
+
+    def _validate(self, deletion_policy: str, name: str):
+        if deletion_policy == DELETE_POLICY:
+            self._add_failure(
+                f"The DeletionPolicy is set to {DELETE_POLICY}. The storage '{name}' will be deleted when you remove "
+                "it from the configuration when performing a cluster update or deleting the cluster.",
+                FailureLevel.INFO,
+            )
+        elif deletion_policy == RETAIN_POLICY:
+            self._add_failure(
+                f"The DeletionPolicy is set to {RETAIN_POLICY}. The storage '{name}' will be retained when you remove "
+                "it from the configuration when performing a cluster update or deleting the cluster.",
+                FailureLevel.INFO,
+            )
+
+
 # --------------- Third party software validators --------------- #
 
 
@@ -878,16 +941,17 @@ class MixedSecurityGroupOverwriteValidator(Validator):
 class _LaunchTemplateValidator(Validator):
     """Abstract class to contain utility functions used by head node and queue LaunchTemplate validators."""
 
+    @staticmethod
     def _build_launch_network_interfaces(
-        self, network_interfaces_count, use_efa, security_group_ids, subnet, use_public_ips=False
+        network_interfaces_count, use_efa, security_group_ids, subnet, use_public_ips=False
     ):
         """Build the needed NetworkInterfaces to launch an instance."""
         network_interfaces = []
-        for device_index in range(network_interfaces_count):
+        for network_interface_index in range(network_interfaces_count):
             network_interfaces.append(
                 {
-                    "DeviceIndex": device_index,
-                    "NetworkCardIndex": device_index,
+                    "DeviceIndex": 0,
+                    "NetworkCardIndex": network_interface_index,
                     "InterfaceType": "efa" if use_efa else "interface",
                     "Groups": security_group_ids,
                     "SubnetId": subnet,
@@ -907,11 +971,7 @@ class _LaunchTemplateValidator(Validator):
             code = e.error_code
             message = str(e)
             subnet_id = kwargs["NetworkInterfaces"][0]["SubnetId"]
-            if code == "UnsupportedOperation":
-                if "does not support specifying CpuOptions" in message:
-                    message.replace("specifying CpuOptions", "disabling simultaneous multithreading")
-                self._add_failure(message, FailureLevel.ERROR)
-            elif code == "InstanceLimitExceeded":
+            if code == "InstanceLimitExceeded":
                 self._add_failure(
                     "You've reached the limit on the number of instances you can run concurrently "
                     f"for the configured instance type. {message}",
@@ -980,13 +1040,6 @@ class HeadNodeLaunchTemplateValidator(_LaunchTemplateValidator):
             if head_node.networking.additional_security_groups:
                 head_node_security_groups.extend(head_node.networking.additional_security_groups)
 
-            # Initialize CpuOptions
-            head_node_cpu_options = (
-                {"CoreCount": head_node.vcpus, "ThreadsPerCore": 1}
-                if head_node.pass_cpu_options_in_launch_template
-                else {}
-            )
-
             head_node_network_interfaces = self._build_launch_network_interfaces(
                 network_interfaces_count=head_node.max_network_interface_count,
                 use_efa=False,  # EFA is not supported on head node
@@ -1001,7 +1054,6 @@ class HeadNodeLaunchTemplateValidator(_LaunchTemplateValidator):
                 MinCount=1,
                 MaxCount=1,
                 ImageId=ami_id,
-                CpuOptions=head_node_cpu_options,
                 NetworkInterfaces=head_node_network_interfaces,
                 DryRun=True,
                 TagSpecifications=self._generate_tag_specifications(tags),
@@ -1045,7 +1097,6 @@ class ComputeResourceLaunchTemplateValidator(_LaunchTemplateValidator):
             if queue.networking.additional_security_groups:
                 queue_security_groups.extend(queue.networking.additional_security_groups)
 
-            # Initialize CpuOptions
             queue_placement_group_id = queue.networking.placement_group.id if queue.networking.placement_group else None
             queue_placement_group = {"GroupName": queue_placement_group_id} if queue_placement_group_id else {}
 
@@ -1056,6 +1107,7 @@ class ComputeResourceLaunchTemplateValidator(_LaunchTemplateValidator):
                 (compute_res for compute_res in queue.compute_resources if compute_res.max_network_interface_count > 1),
                 queue.compute_resources[0],
             )
+            # For SlurmFlexibleComputeResource test only the first InstanceType through a RunInstances
             self._test_compute_resource(
                 compute_resource=dry_run_compute_resource,
                 use_public_ips=bool(queue.networking.assign_public_ip),
@@ -1074,11 +1126,6 @@ class ComputeResourceLaunchTemplateValidator(_LaunchTemplateValidator):
         self, compute_resource, use_public_ips, ami_id, subnet_id, security_groups_ids, placement_group, tags
     ):
         """Test Compute Resource Instance Configuration."""
-        compute_cpu_options = (
-            {"CoreCount": compute_resource.vcpus, "ThreadsPerCore": 1}
-            if compute_resource.disable_simultaneous_multithreading_via_cpu_options
-            else {}
-        )
         network_interfaces = self._build_launch_network_interfaces(
             compute_resource.max_network_interface_count,
             compute_resource.efa.enabled,
@@ -1088,11 +1135,10 @@ class ComputeResourceLaunchTemplateValidator(_LaunchTemplateValidator):
         )
         self._ec2_run_instance(
             availability_zone=AWSApi.instance().ec2.get_subnet_avail_zone(subnet_id),
-            InstanceType=compute_resource.instance_type,
+            InstanceType=compute_resource.instance_types[0],
             MinCount=1,
             MaxCount=1,
             ImageId=ami_id,
-            CpuOptions=compute_cpu_options,
             Placement=placement_group,
             NetworkInterfaces=network_interfaces,
             DryRun=True,
